/*
 nRF Doorbell Camera RX
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 15/11/2015
 Last Modified: 15/11/2015
 
 Check in with the server every 2 seconds, if we receive the system on packet then switch on the mosfet controlling the 
 camera and video transmitter for 32 seconds.
 
 */
 
// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0 Battery ADC
//     LED PB1  3|    |12  PA1 Camera Mosfet
//   Reset PB3  4|    |11  PA2 
//      CE PB2  5|    |10  PA3 
//     CSN PA7  6|    |9   PA4 SCK
//      MI PA6  7|    |8   PA5 MO
//               +----+

#define F_CPU 1000000 // 1 MHz

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include "setup.c"
#include "mirf.c"

int main(void) {
	setup();
	
	uint8_t batteryCheck = 0;
	
	while(1) {
		// Power down the nRF24L01 and sleep for 2 seconds
		NRF_POWERDOWN;
		watchdog_sleep(T2S);
		
		// Check battery voltage every ~2 minute
		if (batteryCheck >= 60) {
			// Check battery voltage
			sbi(ADCSRA, ADEN); // Turn on ADC
			analog_read(batteryAdcPin); // First reading is ignored
			int batteryVoltage = analog_read(batteryAdcPin); 
			cbi(ADCSRA, ADEN); // Turn off ADC
			
			// If battery voltage below 3.5V then turn off
			if (batteryVoltage <= lowVoltageLipo) {
				// Disable permanently
				NRF_POWERDOWN;
				system_sleep();
			}
			batteryCheck = 0;
		}
		batteryCheck++;
		
		// Transmit check in packet
		mirf_transmit_data();
		
		// Wait for packet back
		NRF_RX_POWERUP; // Power up to receiver mode
		mirf_flush_rx_tx(); // Flush RX and TX FIFO
		mirf_CE_hi; // Start listening
		_delay_ms(3);
		mirf_CE_lo; // Stop listening
		
		// Packet received
		if (mirf_status() & (1<<RX_DR)) {
			mirf_CSN_lo; // Pull down chip select
			spi_transfer(R_RX_PAYLOAD); // Send cmd to read rx payload
			uint8_t dataIn = spi_transfer(0x00); // Read payload
			mirf_CSN_hi; // Pull up chip select
			mirf_config_register(STATUS,(1<<RX_DR)); // Reset status register
			
			// If wake up packet received
			if (dataIn == SYSTEM_ON) {
				PORTA |= (1<<cameraMosfet); // Turn on camera/vTX
				NRF_POWERDOWN; // Power down the nRF24L01
				
				// Sleep for 32 seconds
				watchdog_sleep(T8S);
				watchdog_sleep(T8S);
				watchdog_sleep(T8S);
				watchdog_sleep(T8S);
				PORTA &= ~(1<<cameraMosfet);
			}
		}
	}
}

ISR(WDT_vect) { }

ISR(ADC_vect) { }
