/*
 nRF Doorbell Server
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 15/11/2015
 Last Modified: 15/11/2015

 */
 
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define LED PB1
#define speakerMosfetPin PA2

extern void mirf_init(void);
extern void mirf_config(void);

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	USIDR = data;
	USISR = _BV(USIOIF); // clear flag

	while ( (USISR & _BV(USIOIF)) == 0 ) {
		USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	}
	return USIDR;
}

// Write data using SPI
void spi_write_data(uint8_t * dataout, uint8_t len) {
	for (uint8_t i = 0; i < len; i++) {
		spi_transfer(dataout[i]);
	}
}

// Read data using SPI
void spi_read_data(uint8_t * datain, uint8_t len) {
	for (uint8_t i = 0; i < len; i++) {
		datain[i] = spi_transfer(0x00);
	}
}

// Initialise the SPI
void spi_init(void) {
	DDRA |= (1<<PA4); // SPI CLK
	DDRA |= (1<<PA5); // SPI DO
	DDRA &= ~(1<<PA6); // SPI DI
	PORTA |= (1<<PA6); // SPI DI
}

// Sleep
void system_sleep(void) {
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
}

// Setup
void setup(void) {
	_delay_ms(250); // Wait a little bit after power on
	spi_init(); // Initialise SPI
	
	// mIRF initialise and configure
	mirf_init();
	mirf_config();
	
	// Setup outputs
	DDRB |= (1<<LED);
	DDRA |= (1<<speakerMosfetPin);
	
	// Enable Timer0 CTC for the speaker
	OCR0A = 124;
	sbi(TCCR0A, WGM01);
	sbi(TIMSK0, OCIE0A);
	
	// Turn on Timer1 overflow interrupt
	sbi(TIMSK1, TOIE1);
	
	// Turn on pull-up resistors on other ports to save power
	PORTB |= (1<<PB0); 
	PORTA |= ((1<<PA0) | (1<<PA1) | (1<<PA3));
	
	sei(); // Turn on interrupts
}
